@@ -6,6 +6,7 @@
  */
 
 #ifdef WIN32
+#define INCLUDE_WINSOCK
 #include "win32config.h"
 #else
 #include "config.h"
@@ -55,6 +56,9 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_STRINGS_H
+#include <strings.h>
+#endif
 
 #include "xmlmemory.h"
 #include "nanoftp.h"
@@ -263,12 +267,10 @@ xmlNanoFTPScanURL(void *ctx, const char *URL) {
     if (*cur == 0) 
         ctxt->path = xmlMemStrdup("/");
     else {
+        index = 0;
         buf[index] = 0;
-	while (*cur != 0) {
-	    if ((cur[0] == '#') || (cur[0] == '?'))
-	        break;
+	while (*cur != 0)
 	    buf[index++] = *cur++;
-	}
 	buf[index] = 0;
 	ctxt->path = xmlMemStrdup(buf);
     }	
@@ -355,12 +357,10 @@ xmlNanoFTPUpdateURL(void *ctx, const char *URL) {
     if (*cur == 0) 
         ctxt->path = xmlMemStrdup("/");
     else {
+        index = 0;
         buf[index] = 0;
-	while (*cur != 0) {
-	    if ((cur[0] == '#') || (cur[0] == '?'))
-	        break;
+	while (*cur != 0)
 	    buf[index++] = *cur++;
-	}
 	buf[index] = 0;
 	ctxt->path = xmlMemStrdup(buf);
     }	
@@ -649,9 +649,17 @@ xmlNanoFTPSendUser(void *ctx) {
     int res;
 
     if (ctxt->user == NULL)
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "USER anonymous\r\n");
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "USER anonymous\r\n");
+#endif /* HAVE_SNPRINTF */
     else
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "USER %s\r\n", ctxt->user);
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "USER %s\r\n", ctxt->user);
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
     printf(buf);
 #endif
@@ -672,9 +680,17 @@ xmlNanoFTPSendPasswd(void *ctx) {
     int res;
 
     if (ctxt->passwd == NULL)
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "PASS libxml@%s\r\n", hostname);
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "PASS libxml@%s\r\n", hostname);
+#endif /* HAVE_SNPRINTF */
     else
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "PASS %s\r\n", ctxt->passwd);
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "PASS %s\r\n", ctxt->passwd);
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
     printf(buf);
 #endif
@@ -700,7 +716,11 @@ xmlNanoFTPQuit(void *ctx) {
     int len;
     int res;
 
+#ifndef HAVE_SNPRINTF
+    len = sprintf(buf, "QUIT\r\n");
+#else /* HAVE_SNPRINTF */
     len = snprintf(buf, sizeof(buf), "QUIT\r\n");
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
     printf(buf);
 #endif
@@ -820,7 +840,11 @@ xmlNanoFTPConnect(void *ctx) {
 	    /*
 	     * We need proxy auth
 	     */
+#ifndef HAVE_SNPRINTF
+	    len = sprintf(buf, "USER %s\r\n", proxyUser);
+#else /* HAVE_SNPRINTF */
 	    len = snprintf(buf, sizeof(buf), "USER %s\r\n", proxyUser);
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
 	    printf(buf);
 #endif
@@ -837,9 +861,17 @@ xmlNanoFTPConnect(void *ctx) {
 			break;
 		case 3:
 		    if (proxyPasswd != NULL)
+#ifndef HAVE_SNPRINTF
+			len = sprintf(buf, "PASS %s\r\n", proxyPasswd);
+#else /* HAVE_SNPRINTF */
 			len = snprintf(buf, sizeof(buf), "PASS %s\r\n", proxyPasswd);
+#endif /* HAVE_SNPRINTF */
 		    else
+#ifndef HAVE_SNPRINTF
+			len = sprintf(buf, "PASS libxml@%s\r\n",
+#else /* HAVE_SNPRINTF */
 			len = snprintf(buf, sizeof(buf), "PASS libxml@%s\r\n",
+#endif /* HAVE_SNPRINTF */
 			               hostname);
 #ifdef DEBUG_FTP
 		    printf(buf);
@@ -878,7 +910,11 @@ xmlNanoFTPConnect(void *ctx) {
 		/* we will try in seqence */
 	    case 1:
 		/* Using SITE command */
+#ifndef HAVE_SNPRINTF
+		len = sprintf(buf, "SITE %s\r\n", ctxt->hostname);
+#else /* HAVE_SNPRINTF */
 		len = snprintf(buf, sizeof(buf), "SITE %s\r\n", ctxt->hostname);
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
 		printf(buf);
 #endif
@@ -902,10 +938,18 @@ xmlNanoFTPConnect(void *ctx) {
 	    case 2:
 		/* USER user@host command */
 		if (ctxt->user == NULL)
+#ifndef HAVE_SNPRINTF
+		    len = sprintf(buf, "USER anonymous@%s\r\n",
+#else /* HAVE_SNPRINTF */
 		    len = snprintf(buf, sizeof(buf), "USER anonymous@%s\r\n",
+#endif /* HAVE_SNPRINTF */
 			           ctxt->hostname);
 		else
+#ifndef HAVE_SNPRINTF
+		    len = sprintf(buf, "USER %s@%s\r\n",
+#else /* HAVE_SNPRINTF */
 		    len = snprintf(buf, sizeof(buf), "USER %s@%s\r\n",
+#endif /* HAVE_SNPRINTF */
 			           ctxt->user, ctxt->hostname);
 #ifdef DEBUG_FTP
 		printf(buf);
@@ -923,9 +967,17 @@ xmlNanoFTPConnect(void *ctx) {
 		    return(0);
 		}    
 		if (ctxt->passwd == NULL)
+#ifndef HAVE_SNPRINTF
+		    len = sprintf(buf, "PASS libxml@%s\r\n", hostname);
+#else /* HAVE_SNPRINTF */
 		    len = snprintf(buf, sizeof(buf), "PASS libxml@%s\r\n", hostname);
+#endif /* HAVE_SNPRINTF */
 		else
+#ifndef HAVE_SNPRINTF
+		    len = sprintf(buf, "PASS %s\r\n", ctxt->passwd);
+#else /* HAVE_SNPRINTF */
 		    len = snprintf(buf, sizeof(buf), "PASS %s\r\n", ctxt->passwd);
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
 		printf(buf);
 #endif
@@ -990,7 +1042,7 @@ xmlNanoFTPConnect(void *ctx) {
     res = xmlNanoFTPGetResponse(ctxt);
     switch (res) {
 	case 2:
-	    return(0);
+	    break;
 	case 3:
 	    fprintf(stderr, "FTP server asking for ACCNT on anonymous\n");
 	case 1:
@@ -1061,7 +1113,11 @@ xmlNanoFTPCwd(void *ctx, char *directory) {
      *     250
      *     500, 501, 502, 421, 530, 550
      */
+#ifndef HAVE_SNPRINTF
+    len = sprintf(buf, "CWD %s\r\n", directory);
+#else /* HAVE_SNPRINTF */
     len = snprintf(buf, sizeof(buf), "CWD %s\r\n", directory);
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
     printf(buf);
 #endif
@@ -1108,7 +1164,11 @@ xmlNanoFTPGetConnection(void *ctx) {
     dataAddr.sin_family = AF_INET;
 
     if (ctxt->passive) {
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "PASV\r\n");
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "PASV\r\n");
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
 	printf(buf);
 #endif
@@ -1164,7 +1224,11 @@ xmlNanoFTPGetConnection(void *ctx) {
 	}
 	adp = (unsigned char *) &dataAddr.sin_addr;
 	portp = (unsigned char *) &dataAddr.sin_port;
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "PORT %d,%d,%d,%d,%d,%d\r\n",
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "PORT %d,%d,%d,%d,%d,%d\r\n",
+#endif /* HAVE_SNPRINTF */
 	               adp[0] & 0xff, adp[1] & 0xff, adp[2] & 0xff, adp[3] & 0xff,
 		       portp[0] & 0xff, portp[1] & 0xff);
         buf[sizeof(buf) - 1] = 0;
@@ -1363,14 +1427,22 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
         if (xmlNanoFTPCwd(ctxt, ctxt->path) < 1)
 	    return(-1);
 	ctxt->dataFd = xmlNanoFTPGetConnection(ctxt);
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "LIST -L\r\n");
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "LIST -L\r\n");
+#endif /* HAVE_SNPRINTF */
     } else {
 	if (filename[0] != '/') {
 	    if (xmlNanoFTPCwd(ctxt, ctxt->path) < 1)
 		return(-1);
 	}
 	ctxt->dataFd = xmlNanoFTPGetConnection(ctxt);
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "LIST -L %s\r\n", filename);
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "LIST -L %s\r\n", filename);
+#endif /* HAVE_SNPRINTF */
     }
 #ifdef DEBUG_FTP
     printf(buf);
@@ -1462,7 +1534,11 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
 	return(-1);
     ctxt->dataFd = xmlNanoFTPGetConnection(ctxt);
 
+#ifndef HAVE_SNPRINTF
+    len = sprintf(buf, "TYPE I\r\n");
+#else /* HAVE_SNPRINTF */
     len = snprintf(buf, sizeof(buf), "TYPE I\r\n");
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
     printf(buf);
 #endif
@@ -1477,9 +1553,17 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
 	return(-res);
     }
     if (filename == NULL)
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "RETR %s\r\n", ctxt->path);
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "RETR %s\r\n", ctxt->path);
+#endif /* HAVE_SNPRINTF */
     else
+#ifndef HAVE_SNPRINTF
+	len = sprintf(buf, "RETR %s\r\n", filename);
+#else /* HAVE_SNPRINTF */
 	len = snprintf(buf, sizeof(buf), "RETR %s\r\n", filename);
+#endif /* HAVE_SNPRINTF */
 #ifdef DEBUG_FTP
     printf(buf);
 #endif
